<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://lalokalabs.co
 * @since      1.0.0
 *
 * @package    Getotp
 * @subpackage Getotp/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Getotp
 * @subpackage Getotp/includes
 * @author     LaLoka Labs <engineering@lalokalabs.co>
 */
class Getotp
{

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Getotp_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct()
    {
        if (defined('GETOTP_VERSION')) {
            $this->version = GETOTP_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        $this->plugin_name = 'getotp';

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Getotp_Loader. Orchestrates the hooks of the plugin.
     * - Getotp_i18n. Defines internationalization functionality.
     * - Getotp_Admin. Defines all hooks for the admin area.
     * - Getotp_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies()
    {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-getotp-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-getotp-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-getotp-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-getotp-public.php';

        $this->loader = new Getotp_Loader();
    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Getotp_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale()
    {
        $plugin_i18n = new Getotp_i18n();

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks()
    {
        $plugin_admin = new Getotp_Admin($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');

        // WordPress with GETOTP

        $this->loader->add_action('admin_menu', $plugin_admin, 'getotp_add_plugin_page');
        $this->loader->add_action('admin_init', $plugin_admin, 'getotp_page_init');
        
        // Add Settings link to the plugin
        $plugin_basename = plugin_basename(plugin_dir_path(__DIR__) . $this->plugin_name . '.php');

        
        $this->loader->add_filter('plugin_action_links_' . $plugin_basename, $plugin_admin, 'add_action_links');


        $this->loader->add_action('admin_notices', $plugin_admin, 'getotp_admin_error_notice');

        $this->loader->add_action('wp_ajax_my_dismiss_getotp_notice', $plugin_admin, 'my_dismiss_getotp_notice');

        // add mobile number to profile

        $this->loader->add_action('show_user_profile', $plugin_admin, 'show_getotp_profile_fields');
        $this->loader->add_action('edit_user_profile', $plugin_admin, 'show_getotp_profile_fields');

        $this->loader->add_action('personal_options_update', $plugin_admin, 'save_getotp_profile_fields');
        $this->loader->add_action('edit_user_profile_update', $plugin_admin, 'save_getotp_profile_fields');
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks()
    {
        $plugin_public = new Getotp_Public($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');

        // WordPress with GETOTP

        $this->loader->add_action('login_enqueue_scripts', $plugin_public, 'otp_login_scripts', 1);

        $this->loader->add_action('login_form', $plugin_public, 'wp_otp_login_form');
        $this->loader->add_filter('login_message', $plugin_public, 'wp_otp_login_form_message');


        $this->loader->add_filter('authenticate', $plugin_public, 'verify_login_with_otp', 10, 3);

        $this->loader->add_action('wp_loaded', $plugin_public, 'authenticate_otp_user');

        // WooCommerce with GETOTP

        $this->loader->add_action('woocommerce_login_form', $plugin_public, 'woo_otp_login_form');

        // Ajax

        $this->loader->add_action('wp_ajax_getotp_ajax_login_with_otp', $plugin_public, 'ajax_login_with_otp');
        $this->loader->add_action('wp_ajax_nopriv_getotp_ajax_login_with_otp', $plugin_public, 'ajax_login_with_otp');
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run()
    {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name()
    {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    Getotp_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader()
    {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function get_version()
    {
        return $this->version;
    }
}
